/*
*********************************************************************************************************
*                                     Embedded Systems Building Blocks
*                                  Complete and Ready-to-Use Modules in C
*
*                                    Asynchronous Serial Communications
*                                           Buffered Serial I/O 
*                                                 (RTOS)
*
*                            (c) Copyright 1999, Jean J. Labrosse, Weston, FL
*                                           All Rights Reserved
*
* Filename   : COMMRTOS.C
* Programmer : Jean J. Labrosse
* Translated by : Won-Ho, Sung
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                               ũ
*********************************************************************************************************
*/

#include "c:\\IC96\\Work\\ucos_ii\\includes.h"
#include "CommRtos.h"
/*$PAGE*/
/*
*********************************************************************************************************
*                                                 
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                               Ÿ
*********************************************************************************************************
*/

typedef struct {
    INT16U     RingBufRxCtr;                /* Rx  ۿ                              */
    OS_EVENT  *RingBufRxSem;                /*                                     */
    INT8U     *RingBufRxInPtr;              /*  ͸   Ű                */ 
    INT8U     *RingBufRxOutPtr;             /*  о ͸ Ű                   */ 
    INT8U      RingBufRx[COMM_RX_BUF_SIZE]; /* ſ   念(Rx)                             */ 

    INT16U     RingBufTxCtr;                /* Tx  ۿ                              */
    OS_EVENT  *RingBufTxSem;                /* ۽                                    */
    INT8U     *RingBufTxInPtr;              /*  ͸   Ű                */
    INT8U     *RingBufTxOutPtr;             /*  о ͸ Ű                   */
    INT8U      RingBufTx[COMM_TX_BUF_SIZE]; /* ۽ſ   念(Tx)                             */

} COMM_RING_BUF;

/*
*********************************************************************************************************
*                                              
*********************************************************************************************************
*/

COMM_RING_BUF  ComBuf;
COMM_RING_BUF  *pbuf;

void RxIntHandler(void)
{
    INT8U c;

    c = sbuf;                                              /* Process received character               */

    if (pbuf->RingBufRxCtr < COMM_RX_BUF_SIZE) {           /* See if buffer is full                    */
        pbuf->RingBufRxCtr++;                              /* No, increment character count            */
        *pbuf->RingBufRxInPtr++ = c;                       /* Put character into buffer                */
        if (pbuf->RingBufRxInPtr == &pbuf->RingBufRx[COMM_RX_BUF_SIZE]) { /* Wrap IN pointer           */
            pbuf->RingBufRxInPtr = &pbuf->RingBufRx[0];
        }
        OSSemPost(pbuf->RingBufRxSem);                     /* Indicate that character was received     */
    }
}

void TxIntHandler(void)
{
    INT8U c, err;
    
    c = CommGetTxChar(&err);                               /* Get next character to send.               */
    if (err != COMM_TX_EMPTY) {                            /* Do we have anymore characters to send ?   */
        sbuf = c;                                          /* Yes, Send character                       */
    }
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                             б
*
*
*         : α׷  äηκ ͸ о   Լ ȣѴ.
*                Լ äο Ͱ ŵǰų ŸӾƿ    Ѵ.
*
*     : 'ch'    COMM Ʈ ä ȣ.    ´:
*                           COMM1
*                           COMM2
*               'to'    Ͱ ŵǱ⸦ ٸ ŸӾƿ Ŭ ƽ  Ѵ.
*                         0 ϸ Լ Ͱ ŵ    Ѵ.
*
*               'err'   ڵ尡   Ű .    ȴ:
*                           COMM_NO_ERR : ۿ ͸ о  
*                           COMM_RX_TIMEOUT : ŸӾƿ  
*                           COMM_BAD_CH : ȿ ä  
* ϰ      : ۿ о (ŸӾƿ   NUL Ѵ)
*********************************************************************************************************
*/

INT8U  CommGetChar (INT16U to, INT8U *err)
{
    INT8U          c;
    INT8U          oserr;


    OSSemPend(pbuf->RingBufRxSem, to, &oserr);             /* Ͱ ŵǱ⸦ ٸ             */
    if (oserr == OS_TIMEOUT) {                             /* ŸӾƿ Ǿ°?                 */
        *err = COMM_RX_TIMEOUT;                            /* , ڵ带 Ѵ                  */
        return (NUL);
    } else {
        OS_ENTER_CRITICAL();
        pbuf->RingBufRxCtr--;                              /* ƴϿ,  ī͸ ҽŲ       */
        c = *pbuf->RingBufRxOutPtr++;                      /* ۿ ͸ о               */
        if (pbuf->RingBufRxOutPtr == &pbuf->RingBufRx[COMM_RX_BUF_SIZE]) {     /*   ó     */
            pbuf->RingBufRxOutPtr = &pbuf->RingBufRx[0];
        }
        OS_EXIT_CRITICAL();
        *err = COMM_NO_ERR;
        return (c);
    }
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                       ۽    б
*
*
*         : ۽ ISR ۷κ  ۽ ͸ о  ؼ  Լ ȣѴ.
*                Լ ۿ ͸ о  ۰ ٸ FALSE Ѵ.
*                ۽ ISR  Լ FALSE ϸ ۰ ִ  ǹϹǷ
*               ۽ ͷƮ Ȱȭ Ѵ.
*     : 'ch'    COMM Ʈ ä ȣ.    ´:
*                           COMM1
*                           COMM2
*               'err'   ڵ尡   Ű :
*               'err'   ڵ尡   Ű :
*                           COMM_NO_ERR : ϳ ̻ Ͱ ۿ ϴ 
*                           COMM_TX_EMPTY : ۰  ִ 
*                           COMM_BAD_CH : ȿ ä  
* ϰ      : ۽Źۿ о . ۰   NUL Ѵ.
*********************************************************************************************************
*/

INT8U  CommGetTxChar (INT8U *err)
{
    INT8U          c;

    if (pbuf->RingBufTxCtr > 0) {                          /* ۿ Ͱ ִ°?                  */
        pbuf->RingBufTxCtr--;                              /* ,  ī͸ ҽŲ           */
        c = *pbuf->RingBufTxOutPtr++;                      /* ۿ ͸ д´                 */
        if (pbuf->RingBufTxOutPtr == &pbuf->RingBufTx[COMM_TX_BUF_SIZE]) {     /*   ó     */
            pbuf->RingBufTxOutPtr = &pbuf->RingBufTx[0];
        }
        OSSemPost(pbuf->RingBufTxSem);                     /* ۿ Ѱ 念  ˸ */
        *err = COMM_NO_ERR;
        return (c);                                        /*  ͸ о               */
    } else {
        *err = COMM_TX_EMPTY;
        return (NUL);                                      /* ۰                             */
    }
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                          Ÿ ʱȭ
*
*
*         : α׷ Ÿ ʱȭ ϱ   Լ ȣѴ.
*               Ÿ ٸ Լ ȣϱ  ݵ  Լ  ȣؾ Ѵ.
*     : 
*********************************************************************************************************
*/

void  CommInit (void)
{
    INT8U val;

    pbuf                  = &ComBuf;                       /* COMM1    ʱȭ              */
    pbuf->RingBufRxCtr    = 0;
    pbuf->RingBufRxInPtr  = &pbuf->RingBufRx[0];
    pbuf->RingBufRxOutPtr = &pbuf->RingBufRx[0];
    pbuf->RingBufRxSem    = OSSemCreate(0);
    pbuf->RingBufTxCtr    = 0;
    pbuf->RingBufTxInPtr  = &pbuf->RingBufTx[0];
    pbuf->RingBufTxOutPtr = &pbuf->RingBufTx[0];
    pbuf->RingBufTxSem    = OSSemCreate(COMM_TX_BUF_SIZE);

    /* Prevent other bits of ioc1 from being altered */ 
    wsr = 15;
    val = ioc1;
    wsr = 0;
    val |= 0x20;
    
    sp_con = 0x09;     /* mode1 & recieve enable */
    ioc1 = val;        /* select TxD */

    /* Baud rate must be written in the order of LSB first at the same address */
    baud_rate = ((unsigned char) BAUD_REG);
    baud_rate = ((unsigned char) (BAUD_REG >> 8));
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                        ۰  Ȯ
*
*
*         : α׷  äο Ͱ ȿ Ȯϱ   Լ ȣѴ.
*                Ʈ ̻ Ͱ ȿϸ FALSE ϰ ׷   TRUE Ѵ.
*
*     : 'ch'    COMM Ʈ ä ȣ.    ´:
*                           COMM1
*                           COMM2
* ϰ      : TRUE    ۰  
*               FALSE   ۰  ʴ  Ǵ ȿ ä  
*********************************************************************************************************
*/

BOOLEAN  CommIsEmpty (void)
{
    BOOLEAN        empty;

    OS_ENTER_CRITICAL();
    if (pbuf->RingBufRxCtr > 0) {                          /* ۿ Ͱ ִ°?                  */
        empty = FALSE;                                     /* ۰                           */
    } else {
        empty = TRUE;                                      /* ۰                             */
    }
    OS_EXIT_CRITICAL();
    return (empty);
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                    ۽Ź۰  á Ȯ
*
*
*         : α׷ ۽Źۿ ͸    ִ Ȯϱ   Լ ȣѴ.
*               ,  Լ ۽Ź۰  á ȮѴ.
*               ۰   Լ TRUE ϰ ׷   FALSE Ѵ.
*     : 'ch'    COMM Ʈ ä ȣ.    ´:
*                           COMM1
*                           COMM2
* ϰ      : TRUE    ۰  á.
*               FALSE   ۰  ʾҰų ȿä  .
*********************************************************************************************************
*/

BOOLEAN  CommIsFull (void)
{
    BOOLEAN        full;

    OS_ENTER_CRITICAL();
    if (pbuf->RingBufTxCtr < COMM_TX_BUF_SIZE) {           /* ۰ á Ȯ                       */
        full = FALSE;                                      /* ۰                           */
    } else {
        full = TRUE;                                       /* ۰  á                       */
    }
    OS_EXIT_CRITICAL();
    return (full);
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                               
*
*
*         :  Լ äη ͸   ȣȴ. ۰    ۰ 
*                ٸ.  ŸӾƿ   ۰   ȣԼ Ѵ.
*               ŸӾƿ 0 ϸ ۰    ٸ.
*                 ۿ Ǵ Ͱ ۽ ISR  ۵ȴ.
*               ۰   ¿ ù ° Ͱ ۿ   ۽ ISR Ȱȭ ȴ.
*
*
*     : 'ch'    COMM Ʈ ä ȣ.    ´:
*                           COMM1
*                           COMM2
*               'c'     ۿ ϰ ϴ .
*               'to'    ۰     ٸ ϴ Ŭ ƽ  ŸӾƿ .
*                       ŸӾƿ 0 ϸ ۰    ٸ.
* ϰ      : COMM_NO_ERR      Ͱ  ۿ  
*               COMM_TX_TIMEOUT   ŸӾƿ    ۰   
*               COMM_BAD_CH      ȿ ä ȣ  
*********************************************************************************************************
*/

INT8U  CommPutChar (INT8U c, INT16U to)
{
    INT8U          oserr;

    OSSemPend(pbuf->RingBufTxSem, to, &oserr);             /* ۽Źۿ    ٸ     */
    if (oserr == OS_TIMEOUT) {
        return (COMM_TX_TIMEOUT);                          /* ŸӾƿ , ڵ            */
    }
    OS_ENTER_CRITICAL();
    pbuf->RingBufTxCtr++;                                  /*  ī͸ Ų               */
    *pbuf->RingBufTxInPtr++ = c;                           /* ۿ ͸ Ѵ                 */
    if (pbuf->RingBufTxInPtr == &pbuf->RingBufTx[COMM_TX_BUF_SIZE]) {     /*   ó          */
        pbuf->RingBufTxInPtr = &pbuf->RingBufTx[0];
    }
    if (pbuf->RingBufTxCtr == 1) {                         /* ù ° ΰ?                      */
        ipend1 |= 0x01;                                    /* , ۽ ͷƮ Ȱȭ                 */
    }
    OS_EXIT_CRITICAL();
    return (COMM_NO_ERR);
}
