/*
*********************************************************************************************************
*                                   Embedded Systems Building Blocks
*                                Complete and Ready-to-Use Modules in C
*
*                                        LCD Display Module Driver
*
*                            (c) Copyright 1999, Jean J. Labrosse, Weston, FL
*                                           All Rights Reserved
*
* Filename   : LCD.C
* Programmer : Jean J. Labrosse
* Translated by : Won-Ho, Sung
*********************************************************************************************************
*                                                 
*
*
*   ĺ ڸ ÷ Ҽ ִ  ̽ Ѵ
*
* ̹    Ĩ  ϴ  Ĵ޸ LCD Ѵ:
*      HD44780 Ʈ Ʈ LCD Ʈѷ
*
*  ̹     LCD ÷̸ Ѵ:
*
*     1   x 16      2  x 16      4  x 16 
*     1   x 20      2  x 20      4  x 20 
*     1   x 24      2  x 24 
*     1   x 40      2  x 40 
*********************************************************************************************************
*/

/*$PAGE*/
/*
*********************************************************************************************************
*                                              ũ 
*********************************************************************************************************
*/

#include "c:\\IC96\\Work\\ucos_ii\\includes.h"
#include "lcd.h"

/*
*********************************************************************************************************
*                                               
*********************************************************************************************************
*/

                                       /* ----------------------  HD44780 ɾ  -------------------- */
#define  DISP_CMD_CLS            0x01  /* Clr display : ÷ Ŭ, Ŀ Ȩ ġŲ    */
#define  DISP_CMD_FNCT           0x3B  /* Function Set:   8Ʈ, 1/16 Ƽ, 5x8 Ʈ         */
#define  DISP_CMD_MODE           0x06  /* Entry mode  :    ڵ 巹           */
#define  DISP_CMD_ON_OFF         0x0C  /* Disp ON/OFF : ÷ , Ŀ ,          */

/*
*********************************************************************************************************
*                                               
*********************************************************************************************************
*/

static  INT8U       *dataPtr;
static  INT8U       DispMaxCols;       /*  ִ  (1δ  )                         */
static  INT8U       DispMaxRows;       /*  ִ                                                */
static  OS_EVENT   *DispSem;           /* LCD                                                */


const  INT8U       DispBar1[] = {0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10};
const  INT8U       DispBar2[] = {0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18};
const  INT8U       DispBar3[] = {0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C};
const  INT8U       DispBar4[] = {0x1E, 0x1E, 0x1E, 0x1E, 0x1E, 0x1E, 0x1E, 0x1E};
const  INT8U       DispBar5[] = {0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F};

/*
*********************************************************************************************************
*                                             Լ 
*********************************************************************************************************
*/

static  void        DispCursorSet(INT8U row, INT8U col);

/*$PAGE*/
/*
*********************************************************************************************************
*                                             ÷
*
*      : ǥġ   ڸ ÷ Ѵ
*  : 'row'  LCD Ŀ  ġ
*                  'row' 0 'DispMaxRows - 1'  ´
*            'col'  LCD Ŀ  ġ
*                  'col' 0 'DispMaxCols - 1'  ´
*            'c'       ġ ǥõ  
* ϰ   : 
*********************************************************************************************************
*/

void  DispChar (INT8U row, INT8U col, char c)
{
    INT8U err;


    if (row < DispMaxRows && col < DispMaxCols) {
        OSSemPend(DispSem, 0, &err);        /* ǥġ Ÿ ׼ ´                       */
        DispCursorSet(row, col);            /* Ŀ ROW/COL ġŲ                             */
        DispSel(DISP_SEL_DATA_REG);
        DispDataWr(c);                      /* ǥġ ڸ                                   */
        OSSemPost(DispSem);                 /* ׼ Ѵ                                     */
    }
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                                
*
*      :     Ŀ  ó ġ 
*
*  : 'line'    ѹ
*            0 'DispMaxRows - 1'  ´
* ϰ   : 
*********************************************************************************************************
*/

void  DispClrLine (INT8U line)
{
    INT8U i;
    INT8U err;


    if (line < DispMaxRows) {
        OSSemPend(DispSem, 0, &err);             /* ǥġ Ÿ ׼ ´                  */
        DispCursorSet(line, 0);                  /*   ó ġ Ŀ               */
        DispSel(DISP_SEL_DATA_REG);              /* LCD  ͸ Ѵ                   */
        for (i = 0; i < DispMaxCols; i++) {      /*   ġ   ࿡ ' '        */
            DispDataWr(' ');                     /*  ġ ASCII space                    */
        }
        DispCursorSet(line, 0);                  /* Ŀ  ó ġ                    */
        OSSemPost(DispSem);                      /* ׼ Ѵ                                */
    }
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                              ũ 
*
*  : ÷ ü ŬѴ
*    : 
* ϰ     : 
*********************************************************************************************************
*/

void  DispClrScr (void)
{
    INT8U err;


    OSSemPend(DispSem, 0, &err);       /* ǥġ Ÿ ׼ ´                            */
    DispSel(DISP_SEL_CMD_REG);         /* LCD  ͸ Ѵ                               */
    DispDataWr(DISP_CMD_CLS);          /* Ŭ                                             */
    OSTimeDly(2);                      /* 2msec̻ ٸ(ΰ ƽ̸ )                     */
    OSSemPost(DispSem);                /* ׼ Ѵ                                          */
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                     Ŀ  ġ  (ο)
*
*      : Ŀ LCD  ġ 
*  : 'row'  LCD Ŀ  ġ
*                  'row' 0 'DispMaxRows - 1'  ´
*            'col'  LCD Ŀ  ġ
*                  'col' 0 'DispMaxCols - 1'  ´
* ϰ   : 
*********************************************************************************************************
*/

static  void  DispCursorSet (INT8U row, INT8U col)
{
    DispSel(DISP_SEL_CMD_REG);                             /* ɾ                       */
    switch (row) {
        case 0:
             if (DispMaxRows == 1) {                       /* 1  LCD   óѴ    */
                 if (col < (DispMaxCols >> 1)) {
                     DispDataWr(0x80 + col);               /*  ù °  0x80         */
                 } else {                                  /*   °  0xC0         */
                     DispDataWr(0xC0 + col - (DispMaxCols >> 1));
                 }
             } else {
                 DispDataWr(0x80 + col);                   /* LCD 1                          */
             }
             break;

        case 1:
             DispDataWr(0xC0 + col);                       /* LCD 2                          */
             break;

        case 2:
             DispDataWr(0x80 + DispMaxCols + col);         /* LCD 3                          */
             break;

        case 3:
             DispDataWr(0xC0 + DispMaxCols + col);         /* LCD 4                          */
             break;
    }
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                                
*
*      :  ڿ  Ʈ  Ѵ
*  : 'id'  ϰ ϴ Ʈ Ͽ  id
*            'pat' Ʈ   ִ 8Ʈ  
* ϰ   : 
*********************************************************************************************************
*/

void  DispDefChar (INT8U id, INT8U *pat)
{
    INT8U err;
    INT8U i;


    OSSemPend(DispSem, 0, &err);            /* ǥġ Ÿ ׼ ´                       */
    DispSel(DISP_SEL_CMD_REG);              /* ɾ ͸ Ѵ                              */
    DispDataWr(0x40 + (id << 3));           /* CG RAM 巹 ƮѴ                            */
    DispSel(DISP_SEL_DATA_REG);             /*  ͸ Ѵ                              */
    for (i = 0; i < 8; i++) {
        DispDataWr(*pat++);                 /* CG RAM   ִ´                               */
    }
    OSSemPost(DispSem);                     /* ׼ Ѵ                                     */
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                                  Լ
*
*      :  Լ ƹ ϵ  ʴ´.    NOP(No Operation)   ϰ
*              Ŭ CPU Ÿ ҺϹǷ ª  Լ ȴ
*  : 
* ϰ   : 
*********************************************************************************************************
*/

void  DispDummy (void)
{
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                              BAR ǥ
*
*      : LCD⿡  BAR(bar ׷) ׷ش
*  : 'row'  LCD Ŀ  ġ
*                  'row' 0 'DispMaxRows - 1'  ´
*            'val'   BAR  
*                    DispMaxCols * 5 ̻   
* ϰ   : 
* Ʈ     :  Լ Ϸ, DispHorBarInit()  ȣؾ Ѵ
*********************************************************************************************************
*/

void  DispHorBar (INT8U row, INT8U col, INT8U val)
{
    INT8U i;
    INT8U full;
    INT8U fract;
    INT8U err;


    full  = val / 5;                        /*      ON ƴ ȮѴ                   */
    fract = val % 5;                        /*  ġ                                       */
    if (row < DispMaxRows && (col + full - 1) < DispMaxCols) {
        OSSemPend(DispSem, 0, &err);        /* ǥġ Ÿ ׼ ´                       */
        i = 0;                              /* Set counter to limit column to maximum allowable column */
        DispCursorSet(row, col);            /* ٱ׷  ġ Ŀ ġŲ                */
        DispSel(DISP_SEL_DATA_REG);
        while (full > 0) {                  /*                                       */
            DispDataWr(5);                  /*   #5(  )                        */
            i++;                            /* Ʈ ī                                       */
            full--;
        }
        if (fract > 0) {
            DispDataWr(fract);              /*   #'fract'(Ϻθ ä)             */
        }
        OSSemPost(DispSem);                 /* ׼ Ѵ                                     */
    }
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                             BAR ʱȭ
*
*      : ٱ׷  ʱȭѴ  You must
*            DispHorBar() ȣϱ  ݵ  Լ ȣؾ Ѵ
*  : 
* ϰ   : 
*********************************************************************************************************
*/

void  DispHorBarInit (void)
{
    DispDefChar(1, (INT8U *)DispBar1);
    DispDefChar(2, (INT8U *)DispBar2);
    DispDefChar(3, (INT8U *)DispBar3);
    DispDefChar(4, (INT8U *)DispBar4);
    DispDefChar(5, (INT8U *)DispBar5);
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                       ÷ ̹ ʱȭ
*
*      : ÷ ̹ ʱȭѴ
*  : maxrows      LCD ÷ γѹ (1 to 4)
*            maxcols      δ (column) 
* ϰ   : 
* Ʈ     : - DispInit()  Ŀ (ð) ʿ ϹǷ Ƽ½ŷ Ŀ ۵ Ŀ
*              ȣž Ѵ
*            - DispInit() ý ʱȭ  ѹ ȣž Ѵ
*********************************************************************************************************
*/

void  DispInit (INT8U maxrows, INT8U maxcols)
{
    DispInitPort();                    /* ÷ ̹ ϴ I/OƮ ʱȭ Ѵ       */
    DispMaxRows = maxrows;
    DispMaxCols = maxcols;
    DispSem     = OSSemCreate(1);      /* ÷̿                                     */

                                       /* ÷  ʱȭ                                       */
    DispSel(DISP_SEL_CMD_REG);         /* ɾ ͸ Ѵ                                   */
    OSTimeDlyHMSM(0, 0, 0, 50);        /* Ŀ-   15msec̻ ̸ ش(50msec )        */
    DispDataWr(DISP_CMD_FNCT);         /* Function Set:   8Ʈ, 1/16 Ƽ, 5x8 Ʈ         */
    OSTimeDly(2);                      /*  ״  Ȯ ϸ ȵ!                           */
    DispDataWr(DISP_CMD_FNCT);         /*   4 !                                        */
    OSTimeDly(2);                      /*      HD44780  Ʈ                   */
    DispDataWr(DISP_CMD_FNCT);
    OSTimeDly(2);
    DispDataWr(DISP_CMD_FNCT);
    OSTimeDly(2);

    DispDataWr(DISP_CMD_ON_OFF);       /* Disp ON/OFF : ÷ , Ŀ ,          */
    DispDataWr(DISP_CMD_MODE);         /* Entry mode  :    ڵ 巹           */
    DispDataWr(DISP_CMD_CLS);          /* ÷ Ŭ                                  */
    OSTimeDly(2);                      /* ּ 2msec̻  (2 ƽ̸ )                  */
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                         ASCIIڿ ÷
*
*      : ǥġ ASCII ڿ ÷ Ѵ
*  : 'row'  LCD Ŀ  ġ
*                  'row' 0 'DispMaxRows - 1'  ´
*            'col'  LCD Ŀ  ġ
*                  'col' 0 'DispMaxCols - 1'  ´
*            's'   row/col ÷ ϰ ϴ ASCIIڿ
*
* ϰ   : 
*********************************************************************************************************
*/

void  DispStr (INT8U row, INT8U col, char *s)
{
    INT8U i;
    INT8U err;


    if (row < DispMaxRows && col < DispMaxCols) {
        OSSemPend(DispSem, 0, &err);        /* ǥġ Ÿ ׼ ´                       */
        DispCursorSet(row, col);            /* Ŀ ROW/COL ġŲ                             */
        DispSel(DISP_SEL_DATA_REG);
        i = col;                            /* ִ 밡  Ʈ ī͸ ƮѴ         */
        while (i < DispMaxCols && *s) {     /* (str+limit) DispMaxCols ̿ ڸ             */
            DispDataWr(*s++);               /* ǥġ ڸ                                   */
            i++;                            /* Ʈ ī                                       */
        }
        OSSemPost(DispSem);                 /* ׼ Ѵ                                     */
    }
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                     ÷ ġ  
*
*      : ÷ ġ ѹƮ ͸ 
*  : 'data'   
* ϰ   : 
* Ʈ     :   ̿ ּ 40usec ̸ ֱ ؼ DISP_DLY_CNTS(LCD.H)  
*            ʿ䰡  ̴. ڰ 迡    80usec ̰ ʿߴ!
*            LCD ۾ ڰŸų ڰ  ġ ̸ DISP_DLY_CNTS   
*
*********************************************************************************************************
*/

#ifdef ORIGINAL_DRIVER
void  DispDataWr (INT8U data)
{
    INT8U  dly;


    outp(DISP_PORT_DATA, data);                  /* ÷ ⿡ ͸                     */
    outp(DISP_PORT_CMD,  0x01);                  /* E High ƮѴ                            */
    DispDummy();                                 /*  1usec                                     */
    outp(DISP_PORT_CMD,  0x00);                  /* E Low ƮѴ                             */
    for (dly = DISP_DLY_CNTS; dly > 0; dly--) {  /* ּ 40 usec                               */
        DispDummy();
    }
}
#else
void  DispDataWr (INT8U data)
{
    INT8U  dly;


    *dataPtr = data;
    for (dly = DISP_DLY_CNTS; dly > 0; dly--) {  /* ּ 40 usec                               */
        DispDummy();
    }
}
#endif

/*
*********************************************************************************************************
*                                   ÷ ̹ I/OƮ ʱȭ
*
*      : ÷ ̹ ϴ I/OƮ ʱȭ Ѵ
*  : 
* ϰ   : 
*********************************************************************************************************
*/

#ifdef ORIGINAL_DRIVER
void  DispInitPort (void)
{
    outp(DISP_PORT_CMD, 0x82);         /* 0 : A , B Է, C                                */
}
#else
void  DispInitPort (void)
{
    /* DTB-196 8255 Ʈ ʱȭ ʿ䰡 Ƿ  κ ƹ ڵ嵵  ʴ´            */
}

#endif

/*
*********************************************************************************************************
*                                      ɾ/  
*
*      : ͸ Ѵ
*  : 
*********************************************************************************************************
*/

#ifdef ORIGINAL_DRIVER
void  DispSel (INT8U sel)
{
    if (sel == DISP_SEL_CMD_REG) {
        outp(DISP_PORT_CMD, 0x02);     /* ɾ   (RS low)                                */
    } else {
        outp(DISP_PORT_CMD, 0x03);     /*    (RS high)                               */
    }
}
#else
void  DispSel (INT8U sel)
{
    if (sel == DISP_SEL_CMD_REG) {
        dataPtr = (BYTE *)LCD_CW;
    } else {
        dataPtr = (BYTE *)LCD_DW;
    }
}
#endif
